
/**
 ******************************************************************************
 * @file    SerialInterface.h
 * @author  ST
 * @version V1.0.0
 * @date    25 October 2017
 * @brief   Implementation of SerialInterface.
******************************************************************************
 * @attention
 *
 * <h2><center>&copy; COPYRIGHT(c) 2017 STMicroelectronics</center></h2>
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *   1. Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright notice,
 *      this list of conditions and the following disclaimer in the documentation
 *      and/or other materials provided with the distribution.
 *   3. Neither the name of STMicroelectronics nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ******************************************************************************
 */

/* Prevent recursive inclusion -----------------------------------------------*/
#ifndef _SERIALINTERFACE_H
#define _SERIALINTERFACE_H

/* Includes ------------------------------------------------------------------*/

#include <string>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/time.h>

#include "mbed.h"
#include "Callback.h"

#include "us_ticker_api.h"

#include "Flasher.h"
#include "SerialBuffer.h"
#include "ISerialInterface.h"

#include "jerryscript-mbed-event-loop/EventLoop.h"

using namespace std;

/* RAW SERIAL check ----------------------------------------------------------*/
#ifndef JSMBED_USE_RAW_SERIAL
#error "Macro 'JSMBED_USE_RAW_SERIAL' not defined, required by SerialInterface"
#else
extern RawSerial pc;
#endif

/* Class Declaration ---------------------------------------------------------*/

/**
 * SerialInterface class which helps reading from terminal through serial port.
 */
class SerialInterface : public ISerialInterface {
public:

    /* Constructor. */
    SerialInterface();
    
    /* Public functions. */
    void printJustHappened();

private:
    /* SerialInterface interface. */
    static ISerialInterface* serialInterface;
    
    /* Functions. */
    void callback();
    void addToBuffer(char c);
    void addCharacter(char c);
    void addSpecialCharacter(char c);
    void handleBackspace();
    void runBuffer() ;
    void flashBuffer();
    bool jerry_port_console_printing;
    void jerry_port_console (const char *format, ...);
    
private:
    SerialBuffer buffer;
    bool inControlChar = false;
    vector<char> controlSequence;
    vector<string> history;
    size_t historyPosition;
};

#endif // _SERIALINTERFACE_H